<?php
/**
 * Login Dongle Plugin
 * 
 * @author Andrea Ercolino, http://notelog.com
 */

class LoginDonglePlugin
{
    const OPTION_NAME = 'login_dongle';                //used in the db
    const CHALLENGE_PREFIX = 'logindongle_';           //to prevent field clashes
    
    /**
     * @var LoginDongleOption
     */
    protected $option;
    
    /**
     * Constructor
     * 
     * This binds events to handlers
     */
    public function __construct()
    {
        load_plugin_textdomain( self::OPTION_NAME, '', basename(dirname(LOGIN_DONGLE_FILENAME)) . '/translations' );
        
        require_once 'LoginDongleOption.php';
        $this->option = new LoginDongleOption();
        
        require_once 'LoginDongleUpdater.php';
        new LoginDongleUpdater();
        
        require_once 'LoginDongleSiteAdmin.php';
        new LoginDongleSiteAdmin();
        
        require_once 'LoginDongleUserAdmin.php';
        new LoginDongleUserAdmin();
        
        add_action('login_enqueue_scripts', array($this, 'handleGET'));
        add_action('login_init', array($this, 'handlePOST'), 0); //highest priority
    }
    
    /**
     * Loads the jQuery library into the login page, so the bookmarklet can use it.
     * 
     * This method is a handler bound to the login_enqueue_scripts action.
     */
    public function handleGET()
    {
        wp_enqueue_script('jquery');
    }
    
    /**
     * Validates POST data. 
     * If the challenge/response field is not properly set, it dies with the configured message.
     * 
     * This method is a handler bound to the login_init action.
     * 
     * WARNING
     * Since version 1.3.0 this method dies only if the POST user has configured a login dongle.
     * Up to version 1.2.2 this method died only if the SITE admin had configured a login dongle.
     */
    public function handlePOST()
    {
        if ('POST' != $_SERVER['REQUEST_METHOD'])
    	{
    		return; //if the request is not a POST
    	}
    	$login_url   = self::scriptUrl(wp_login_url());
    	$current_url = self::scriptUrl($_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
    	if ($current_url != $login_url)
        {
            return; //if the page is not the login
        }
        $username = sanitize_user(isset($_POST['log']) ? $_POST['log'] : $_POST['user_login']);
    	if (empty($username))
    	{
    	    return; //if a username is not given
    	}
    	$userdata = $this->option->get($username);
    	if (! isset($userdata['challenge']) || empty($userdata['challenge']))
    	{
    	    return; //if a challenge is not configured
    	}
    	//wp_magic_quotes() is called by WP from settings.php for having uniformly escaped values
    	//over servers with magic quotes both on and off, BUT it 'forgets' to do the same for keys,
    	//so before comparing Login Dongle db options against $_POST contents, I need to add 
    	//slashes to the response always, but to the challenge only if magic quotes are really on
    	$challenge = self::challengeFieldName($userdata['challenge']);
    	if (get_magic_quotes_gpc())
    	{
    	    $challenge = addslashes($challenge);
    	}
    	$response  = addslashes($userdata['response']);
    	$isValid = isset($_POST[$challenge]) && $_POST[$challenge] == $response;
    	if (! $isValid) 
    	{ 
    	    $sitedata = $this->option->get();
    		wp_die($sitedata['message']);
    	}
    	unset($_POST[$challenge], $_REQUEST[$challenge]);
    }
    
    /**
     * For some reason, a space is correctly urlencoded and sent as a plus by the browser but I get 
     * it in the $_POST as an underscore. (ModSecurity?) To solve the problem and prevent PHP from 
     * misinterpreting other special field chars like [], I'm using encodeFieldName.
     * 
     * @param string $challenge
     * @return string
     */
    static public function challengeFieldName($challenge)
    {
        $result = self::encodeFieldName(self::CHALLENGE_PREFIX . $challenge);
        return $result;
    }
    
    /**
     * Returns an encoded version of the given $decoded field name. 
     * The encoding is very similar to the result of the javascript encodeURIComponent() function.
     * 
     * @param string $decoded
     * @return string
     */
    static protected function encodeFieldName($decoded)
    {
        //exceptions
        //-_.~      <-- http://www.php.net/manual/en/function.rawurlencode.php
        //-_.~!*'() <-- https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/encodeURIComponent
        
        //exceptions to exceptions
        //for some reason, not only each + gets converted to an underscore but also each dot
         
        $result = rawurlencode($decoded);
        $result = str_replace(
            array('%21', '%2A', '%27', '%28', '%29',   '.'), 
            array(  '!',   '*',   "'",   '(',   ')', '%2E'), 
            $result);
        return $result;
    }
    
    /**
     * Writes the given $data value to the given $file name.
     * Used for development.
     * 
     * @param mixed $data
     * @param string $file
     */
    static public function logToFile($data, $file = 'log')
    {
        $data = date('Y-m-d H:i:s') . ' - ' . var_export($data, true);
        if (! $file)
        {
            $file = uniqid('log_');
        }
        $file = basename($file, '.txt');
        file_put_contents(dirname(__FILE__) . "/$file.txt", "\n\n$data", FILE_APPEND);
    }
    
    /**
     * Returns the part of the given $url from the host name up to before the query string
     * 
     * @param string $url
     * @return string
     */
    static public function scriptUrl( $url )
    {
        $result = preg_replace('@^https?://@', '', $url);
    	$result = preg_replace('@\?.*@', '', $result);
    	return $result;
    }
    
    /**
     * Uninstalls this plugin, cleaning up all data
     * 
     */
    static public function uninstall()
    {
        require_once 'LoginDongleOption.php';
        $option = new LoginDongleOption();
        $option->remove_all();
    }
}
